@@ -50,30 +50,42 @@ gsd_should_ignore_unix_mount (GUnixMountEntry *mount)
                 "auto",
                 "autofs",
                 "autofs4",
+                "cgroup",
                 "cifs",
+                "configfs",
                 "cxfs",
+                "debugfs",
                 "devfs",
                 "devpts",
+                "devtmpfs",
                 "ecryptfs",
                 "fdescfs",
+                "fusectl",
                 "gfs",
                 "gfs2",
                 "gpfs",
+                "hugetlbfs",
                 "kernfs",
                 "linprocfs",
                 "linsysfs",
                 "lustre",
                 "lustre_lite",
+                "mfs",
+                "mqueue",
                 "ncpfs",
                 "nfs",
                 "nfs4",
                 "nfsd",
+                "nullfs",
                 "ocfs2",
+                "overlay",
                 "proc",
                 "procfs",
+                "pstore",
                 "ptyfs",
                 "rootfs",
                 "rpc_pipefs",
+                "securityfs",
                 "selinuxfs",
                 "smbfs",
                 "sysfs",
