@@ -3,7 +3,7 @@
  *  gnome-bluetooth - Bluetooth integration for GNOME
  *
  *  Copyright (C) 2012  Bastien Nocera <hadess@hadess.net>
- *
+ *  Copyright © 2017 Endless Mobile, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
+#include <sys/ioctl.h>
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -43,101 +44,60 @@ enum {
 	LAST_SIGNAL
 };
 
+enum {
+	PROP_RFKILL_INPUT_INHIBITED = 1
+};
+
 static int signals[LAST_SIGNAL] = { 0 };
 
-#define CC_RFKILL_GLIB_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
-				CC_RFKILL_TYPE_GLIB, CcRfkillGlibPrivate))
+struct _CcRfkillGlib {
+	GObject parent;
 
-struct CcRfkillGlibPrivate {
 	GOutputStream *stream;
 	GIOChannel *channel;
 	guint watch_id;
 
-	/* Pending Bluetooth enablement */
+	/* rfkill-input inhibitor */
+	int noinput_fd;
+
+	/* Pending Bluetooth enablement.
+	 * If (@change_all_timeout_id != 0), then (task != NULL). The converse
+	 * does not necessarily hold. */
 	guint change_all_timeout_id;
-	struct rfkill_event *event;
-	GSimpleAsyncResult *simple;
-	GCancellable *cancellable;
+	GTask *task;
 };
 
-G_DEFINE_TYPE(CcRfkillGlib, cc_rfkill_glib, G_TYPE_OBJECT)
+G_DEFINE_TYPE (CcRfkillGlib, cc_rfkill_glib, G_TYPE_OBJECT)
 
 #define CHANGE_ALL_TIMEOUT 500
 
 static const char *type_to_string (unsigned int type);
 
-gboolean
-cc_rfkill_glib_send_event_finish (CcRfkillGlib  *rfkill,
-				  GAsyncResult  *res,
-				  GError       **error)
-{
-	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
-
-	g_return_val_if_fail (RFKILL_IS_GLIB (rfkill), FALSE);
-	g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == cc_rfkill_glib_send_event);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return FALSE;
-
-	return (g_simple_async_result_get_op_res_gssize (simple) >= 0);
-}
-
 static void
-write_done_cb (GObject      *source_object,
-	       GAsyncResult *res,
-	       gpointer      user_data)
+cancel_current_task (CcRfkillGlib *rfkill)
 {
-	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
-	GError *error = NULL;
-	gssize ret;
+	if (rfkill->task != NULL) {
+		g_cancellable_cancel (g_task_get_cancellable (rfkill->task));
+		g_clear_object (&rfkill->task);
+	}
 
-	ret = g_output_stream_write_finish (G_OUTPUT_STREAM (source_object), res, &error);
-	if (ret < 0)
-		g_simple_async_result_take_error (simple, error);
-	else
-		g_simple_async_result_set_op_res_gssize (simple, ret);
-
-	g_simple_async_result_complete_in_idle (simple);
-	g_object_unref (simple);
-}
-
-void
-cc_rfkill_glib_send_event (CcRfkillGlib        *rfkill,
-			   struct rfkill_event *event,
-			   GCancellable        *cancellable,
-			   GAsyncReadyCallback  callback,
-			   gpointer             user_data)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_if_fail (RFKILL_IS_GLIB (rfkill));
-	g_return_if_fail (rfkill->priv->stream);
-
-	simple = g_simple_async_result_new (G_OBJECT (rfkill),
-					    callback,
-					    user_data,
-					    cc_rfkill_glib_send_event);
-
-	g_output_stream_write_async (rfkill->priv->stream,
-				     event, sizeof(struct rfkill_event),
-				     G_PRIORITY_DEFAULT,
-				     cancellable, write_done_cb, simple);
+	if (rfkill->change_all_timeout_id != 0) {
+		g_source_remove (rfkill->change_all_timeout_id);
+		rfkill->change_all_timeout_id = 0;
+	}
 }
 
+/* Note that this can return %FALSE without setting @error. */
 gboolean
 cc_rfkill_glib_send_change_all_event_finish (CcRfkillGlib        *rfkill,
 					     GAsyncResult        *res,
 					     GError             **error)
 {
-	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
+	g_return_val_if_fail (CC_RFKILL_IS_GLIB (rfkill), FALSE);
+	g_return_val_if_fail (g_task_is_valid (res, rfkill), FALSE);
+	g_return_val_if_fail (g_async_result_is_tagged (res, cc_rfkill_glib_send_change_all_event), FALSE);
 
-	g_return_val_if_fail (RFKILL_IS_GLIB (rfkill), FALSE);
-	g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == cc_rfkill_glib_send_change_all_event);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return FALSE;
-
-	return g_simple_async_result_get_op_res_gboolean (simple);
+	return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
@@ -145,40 +105,41 @@ write_change_all_again_done_cb (GObject      *source_object,
 				GAsyncResult *res,
 				gpointer      user_data)
 {
-	CcRfkillGlib *rfkill = user_data;
-	GError *error = NULL;
+	g_autoptr(GTask) task = G_TASK (user_data);
+	CcRfkillGlib *rfkill = g_task_get_source_object (task);
+	g_autoptr(GError) error = NULL;
 	gssize ret;
 
 	g_debug ("Finished writing second RFKILL_OP_CHANGE_ALL event");
 
 	ret = g_output_stream_write_finish (G_OUTPUT_STREAM (source_object), res, &error);
 	if (ret < 0)
-		g_simple_async_result_take_error (rfkill->priv->simple, error);
+		g_task_return_error (task, g_steal_pointer (&error));
 	else
-		g_simple_async_result_set_op_res_gboolean (rfkill->priv->simple, ret >= 0);
+		g_task_return_boolean (task, ret >= 0);
 
-	g_simple_async_result_complete_in_idle (rfkill->priv->simple);
-	g_clear_object (&rfkill->priv->simple);
-	g_clear_pointer (&rfkill->priv->event, g_free);
+	/* If this @task has been cancelled, it may have been superceded. */
+	if (rfkill->task == task)
+		g_clear_object (&rfkill->task);
 }
 
 static gboolean
 write_change_all_timeout_cb (CcRfkillGlib *rfkill)
 {
-	g_assert (rfkill->priv->event);
+	struct rfkill_event *event;
+
+	g_assert (rfkill->task != NULL);
 
 	g_debug ("Sending second RFKILL_OP_CHANGE_ALL timed out");
 
-	g_simple_async_result_set_error (rfkill->priv->simple,
-					 G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-					 "Enabling rfkill for %s timed out",
-					 type_to_string (rfkill->priv->event->type));
-	g_simple_async_result_complete_in_idle (rfkill->priv->simple);
+	event = g_task_get_task_data (rfkill->task);
+	g_task_return_new_error (rfkill->task,
+				 G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
+				 "Enabling rfkill for %s timed out",
+				 type_to_string (event->type));
 
-	g_clear_object (&rfkill->priv->simple);
-	g_clear_pointer (&rfkill->priv->event, g_free);
-	g_clear_object (&rfkill->priv->cancellable);
-	rfkill->priv->change_all_timeout_id = 0;
+	g_clear_object (&rfkill->task);
+	rfkill->change_all_timeout_id = 0;
 
 	return G_SOURCE_REMOVE;
 }
@@ -188,33 +149,36 @@ write_change_all_done_cb (GObject      *source_object,
 			  GAsyncResult *res,
 			  gpointer      user_data)
 {
-	CcRfkillGlib *rfkill = user_data;
-	GError *error = NULL;
+	g_autoptr(GTask) task = G_TASK (user_data);
+	CcRfkillGlib *rfkill = g_task_get_source_object (task);
+	g_autoptr(GError) error = NULL;
 	gssize ret;
+	struct rfkill_event *event;
 
 	g_debug ("Sending original RFKILL_OP_CHANGE_ALL event done");
 
+	event = g_task_get_task_data (task);
 	ret = g_output_stream_write_finish (G_OUTPUT_STREAM (source_object), res, &error);
 	if (ret < 0) {
-		g_simple_async_result_take_error (rfkill->priv->simple, error);
+		g_task_return_error (task, g_steal_pointer (&error));
 		goto bail;
-	} else if (rfkill->priv->event->soft == 1 ||
-		   rfkill->priv->event->type != RFKILL_TYPE_BLUETOOTH) {
-		g_simple_async_result_set_op_res_gboolean (rfkill->priv->simple, ret >= 0);
+	} else if (event->soft == 1 ||
+		   event->type != RFKILL_TYPE_BLUETOOTH) {
+		g_task_return_boolean (task, ret >= 0);
 		goto bail;
 	}
 
-	rfkill->priv->change_all_timeout_id = g_timeout_add (CHANGE_ALL_TIMEOUT,
-							     (GSourceFunc) write_change_all_timeout_cb,
-							     rfkill);
+	g_assert (rfkill->change_all_timeout_id == 0);
+	rfkill->change_all_timeout_id = g_timeout_add (CHANGE_ALL_TIMEOUT,
+						       (GSourceFunc) write_change_all_timeout_cb,
+						       rfkill);
 
 	return;
 
 bail:
-	g_simple_async_result_complete_in_idle (rfkill->priv->simple);
-	g_clear_object (&rfkill->priv->simple);
-	g_clear_pointer (&rfkill->priv->event, g_free);
-	g_clear_object (&rfkill->priv->cancellable);
+	/* If this @task has been cancelled, it may have been superceded. */
+	if (rfkill->task == task)
+		g_clear_object (&rfkill->task);
 }
 
 void
@@ -225,37 +189,34 @@ cc_rfkill_glib_send_change_all_event (CcRfkillGlib        *rfkill,
 				      GAsyncReadyCallback  callback,
 				      gpointer             user_data)
 {
-	GSimpleAsyncResult *simple;
+	g_autoptr(GTask) task = NULL;
 	struct rfkill_event *event;
 
-	g_return_if_fail (RFKILL_IS_GLIB (rfkill));
-	g_return_if_fail (rfkill->priv->stream);
+	g_return_if_fail (CC_RFKILL_IS_GLIB (rfkill));
+	g_return_if_fail (rfkill->stream);
 
-	simple = g_simple_async_result_new (G_OBJECT (rfkill),
-					    callback,
-					    user_data,
-					    cc_rfkill_glib_send_change_all_event);
+	task = g_task_new (rfkill, cancellable, callback, user_data);
+	g_task_set_source_tag (task, cc_rfkill_glib_send_change_all_event);
 
-	if (rfkill->priv->change_all_timeout_id > 0) {
-		g_source_remove (rfkill->priv->change_all_timeout_id);
-		rfkill->priv->change_all_timeout_id = 0;
-		write_change_all_timeout_cb (rfkill);
-	}
+	/* Clear any previous task. */
+	cancel_current_task (rfkill);
+	g_assert (rfkill->task == NULL);
 
+	/* Start writing out a new event. */
 	event = g_new0 (struct rfkill_event, 1);
 	event->op = RFKILL_OP_CHANGE_ALL;
 	event->type = rfkill_type;
 	event->soft = enable ? 1 : 0;
 
-	rfkill->priv->event = event;
-	rfkill->priv->simple = simple;
-	rfkill->priv->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	rfkill->priv->change_all_timeout_id = 0;
+	g_task_set_task_data (task, event, g_free);
+	rfkill->task = g_object_ref (task);
+	rfkill->change_all_timeout_id = 0;
 
-	g_output_stream_write_async (rfkill->priv->stream,
+	g_output_stream_write_async (rfkill->stream,
 				     event, sizeof(struct rfkill_event),
 				     G_PRIORITY_DEFAULT,
-				     cancellable, write_change_all_done_cb, rfkill);
+				     cancellable, write_change_all_done_cb,
+				     g_object_ref (task));
 }
 
 static const char *
@@ -334,17 +295,22 @@ emit_changed_signal_and_free (CcRfkillGlib *rfkill,
 		       signals[CHANGED],
 		       0, events);
 
-	if (rfkill->priv->change_all_timeout_id > 0 &&
+	if (rfkill->change_all_timeout_id > 0 &&
 	    got_change_event (events)) {
+		struct rfkill_event *event;
+
 		g_debug ("Received a change event after a RFKILL_OP_CHANGE_ALL event, re-sending RFKILL_OP_CHANGE_ALL");
 
-		g_output_stream_write_async (rfkill->priv->stream,
-					     rfkill->priv->event, sizeof(struct rfkill_event),
+		event = g_task_get_task_data (rfkill->task);
+		g_output_stream_write_async (rfkill->stream,
+					     event, sizeof(struct rfkill_event),
 					     G_PRIORITY_DEFAULT,
-					     rfkill->priv->cancellable, write_change_all_again_done_cb, rfkill);
+					     g_task_get_cancellable (rfkill->task),
+					     write_change_all_again_done_cb,
+					     g_object_ref (rfkill->task));
 
-		g_source_remove (rfkill->priv->change_all_timeout_id);
-		rfkill->priv->change_all_timeout_id = 0;
+		g_source_remove (rfkill->change_all_timeout_id);
+		rfkill->change_all_timeout_id = 0;
 	}
 
 	g_list_free_full (events, g_free);
@@ -398,37 +364,33 @@ event_cb (GIOChannel   *source,
 static void
 cc_rfkill_glib_init (CcRfkillGlib *rfkill)
 {
-	CcRfkillGlibPrivate *priv;
-
-	priv = CC_RFKILL_GLIB_GET_PRIVATE (rfkill);
-	rfkill->priv = priv;
+	rfkill->noinput_fd = -1;
 }
 
-int
-cc_rfkill_glib_open (CcRfkillGlib *rfkill)
+gboolean
+cc_rfkill_glib_open (CcRfkillGlib  *rfkill,
+                     GError       **error)
 {
-	CcRfkillGlibPrivate *priv;
 	int fd;
 	int ret;
 	GList *events;
 
-	g_return_val_if_fail (RFKILL_IS_GLIB (rfkill), -1);
-	g_return_val_if_fail (rfkill->priv->stream == NULL, -1);
-
-	priv = rfkill->priv;
+	g_return_val_if_fail (CC_RFKILL_IS_GLIB (rfkill), FALSE);
+	g_return_val_if_fail (rfkill->stream == NULL, FALSE);
 
 	fd = open("/dev/rfkill", O_RDWR);
 	if (fd < 0) {
-		if (errno == EACCES)
-			g_warning ("Could not open RFKILL control device, please verify your installation");
-		return fd;
+		g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno),
+		                     "Could not open RFKILL control device, please verify your installation");
+		return FALSE;
 	}
 
 	ret = fcntl(fd, F_SETFL, O_NONBLOCK);
 	if (ret < 0) {
-		g_debug ("Can't set RFKILL control device to non-blocking");
+		g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno),
+		                     "Can't set RFKILL control device to non-blocking");
 		close(fd);
-		return ret;
+		return FALSE;
 	}
 
 	events = NULL;
@@ -463,41 +425,135 @@ cc_rfkill_glib_open (CcRfkillGlib *rfkill)
 	}
 
 	/* Setup monitoring */
-	priv->channel = g_io_channel_unix_new (fd);
-	priv->watch_id = g_io_add_watch (priv->channel,
-					 G_IO_IN | G_IO_HUP | G_IO_ERR,
-					 (GIOFunc) event_cb,
-					 rfkill);
+	rfkill->channel = g_io_channel_unix_new (fd);
+	rfkill->watch_id = g_io_add_watch (rfkill->channel,
+					   G_IO_IN | G_IO_HUP | G_IO_ERR,
+					   (GIOFunc) event_cb,
+					   rfkill);
 
-	events = g_list_reverse (events);
-	emit_changed_signal_and_free (rfkill, events);
+	if (events) {
+		events = g_list_reverse (events);
+		emit_changed_signal_and_free (rfkill, events);
+	} else {
+		g_debug ("No rfkill device available on startup");
+	}
 
 	/* Setup write stream */
-	priv->stream = g_unix_output_stream_new (fd, TRUE);
+	rfkill->stream = g_unix_output_stream_new (fd, TRUE);
 
-	return fd;
+	return TRUE;
+}
+
+#define RFKILL_INPUT_INHIBITED(rfkill) (rfkill->noinput_fd >= 0)
+
+gboolean
+cc_rfkill_glib_get_rfkill_input_inhibited (CcRfkillGlib        *rfkill)
+{
+	g_return_val_if_fail (CC_RFKILL_IS_GLIB (rfkill), FALSE);
+
+	return RFKILL_INPUT_INHIBITED(rfkill);
+}
+
+void
+cc_rfkill_glib_set_rfkill_input_inhibited (CcRfkillGlib *rfkill,
+					   gboolean      inhibit)
+{
+	g_return_if_fail (CC_RFKILL_IS_GLIB (rfkill));
+
+	/* Nothing to do if the states already match. */
+	if (RFKILL_INPUT_INHIBITED(rfkill) == inhibit)
+		return;
+
+	if (!inhibit && RFKILL_INPUT_INHIBITED(rfkill)) {
+		close (rfkill->noinput_fd);
+		rfkill->noinput_fd = -1;
+
+		g_debug ("Closed rfkill noinput FD.");
+	}
+
+	if (inhibit && !RFKILL_INPUT_INHIBITED(rfkill)) {
+		int fd, res;
+		/* Open write only as we don't want to do any IO to it ever. */
+		fd = open ("/dev/rfkill", O_WRONLY);
+		if (fd < 0) {
+			if (errno == EACCES)
+				g_warning ("Could not open RFKILL control device, please verify your installation");
+			else
+				g_debug ("Could not open RFKILL control device: %s", g_strerror (errno));
+			return;
+		}
+
+		res = ioctl (fd, RFKILL_IOCTL_NOINPUT, (long) 0);
+		if (res != 0) {
+			g_warning ("Could not disable kernel handling of RFKILL related keys: %s", g_strerror (errno));
+			close (fd);
+			return;
+		}
+
+		g_debug ("Opened rfkill-input inhibitor.");
+
+		rfkill->noinput_fd = fd;
+	}
+
+	g_object_notify (G_OBJECT (rfkill), "rfkill-input-inhibited");
+}
+
+static void
+cc_rfkill_glib_set_property (GObject      *object,
+			 guint	       prop_id,
+			 const GValue *value,
+			 GParamSpec   *pspec)
+{
+	CcRfkillGlib *rfkill = CC_RFKILL_GLIB (object);
+
+	switch (prop_id) {
+	case PROP_RFKILL_INPUT_INHIBITED:
+		cc_rfkill_glib_set_rfkill_input_inhibited (rfkill, g_value_get_boolean (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+cc_rfkill_glib_get_property (GObject    *object,
+			 guint	     prop_id,
+			 GValue	    *value,
+			 GParamSpec *pspec)
+{
+	CcRfkillGlib *rfkill = CC_RFKILL_GLIB (object);
+
+	switch (prop_id) {
+	case PROP_RFKILL_INPUT_INHIBITED:
+		g_value_set_boolean (value, RFKILL_INPUT_INHIBITED(rfkill));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
 cc_rfkill_glib_finalize (GObject *object)
 {
-	CcRfkillGlib *rfkill;
-	CcRfkillGlibPrivate *priv;
+	CcRfkillGlib *rfkill = CC_RFKILL_GLIB (object);
 
-	rfkill = CC_RFKILL_GLIB (object);
-	priv = rfkill->priv;
-
-	if (priv->change_all_timeout_id > 0)
-		write_change_all_timeout_cb (rfkill);
+	cancel_current_task (rfkill);
 
 	/* cleanup monitoring */
-	if (priv->watch_id > 0) {
-		g_source_remove (priv->watch_id);
-		priv->watch_id = 0;
-		g_io_channel_shutdown (priv->channel, FALSE, NULL);
-		g_io_channel_unref (priv->channel);
+	if (rfkill->watch_id > 0) {
+		g_source_remove (rfkill->watch_id);
+		rfkill->watch_id = 0;
+		g_io_channel_shutdown (rfkill->channel, FALSE, NULL);
+		g_io_channel_unref (rfkill->channel);
+	}
+	g_clear_object (&rfkill->stream);
+
+	if (RFKILL_INPUT_INHIBITED(rfkill)) {
+		close (rfkill->noinput_fd);
+		rfkill->noinput_fd = -1;
 	}
-	g_clear_object (&priv->stream);
 
 	G_OBJECT_CLASS(cc_rfkill_glib_parent_class)->finalize(object);
 }
@@ -507,14 +563,23 @@ cc_rfkill_glib_class_init(CcRfkillGlibClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *) klass;
 
-	g_type_class_add_private(klass, sizeof(CcRfkillGlibPrivate));
+	object_class->set_property = cc_rfkill_glib_set_property;
+	object_class->get_property = cc_rfkill_glib_get_property;
 	object_class->finalize = cc_rfkill_glib_finalize;
 
+	g_object_class_install_property (object_class,
+					 PROP_RFKILL_INPUT_INHIBITED,
+					 g_param_spec_boolean ("rfkill-input-inhibited",
+							       "Rfkill input inhibited",
+							       "Whether to prevent the kernel from handling RFKILL related key events.",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	signals[CHANGED] =
 		g_signal_new ("changed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (CcRfkillGlibClass, changed),
+			      0,
 			      NULL, NULL,
 			      NULL,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
