@@ -17,20 +17,17 @@
  *
  */
 
+#include "gsd-wacom-oled-constants.h"
+#include "gsd-device-manager.h"
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 #ifndef __GSD_WACOM_OLED_H
 #define __GSD_WACOM_OLED_H
 
 G_BEGIN_DECLS
 
-/* OLED parameters */
-#define OLED_WIDTH		64			/*Width of OLED icon - hardware dependent*/
-#define OLED_HEIGHT		32			/*Height of OLED icon - hardware dependent*/
-#define LABEL_SIZE		30			/*Maximum length of text for OLED icon*/
-#define MAX_TOKEN		(LABEL_SIZE >> 1)	/*Maximum number of tokens equals half of maximum number of characters*/
-#define MAX_IMAGE_SIZE		1024			/*Size of buffer for storing OLED image*/
-#define MAX_1ST_LINE_LEN	10			/*Maximum number of characters in 1st line of OLED icon*/
-
-void set_oled (GsdWacomDevice *device, char *button_id, char *label);
+gboolean set_oled (GsdDevice *device, guint button, char *label, GError **error);
 char *gsd_wacom_oled_gdkpixbuf_to_base64 (GdkPixbuf *pixbuf);
 
 G_END_DECLS
